import { InjectionToken, ɵɵinject, ɵɵdefineInjectable, ɵsetClassMetadata, Injectable, Inject, ɵɵelementStart, ɵɵtext, ɵɵelementEnd, ɵɵnextContext, ɵɵadvance, ɵɵtextInterpolate, ɵɵdirectiveInject, ɵɵdefineComponent, ɵɵtemplate, ɵɵlistener, ɵɵproperty, ɵɵpureFunction1, Component, Input, ɵɵdefineNgModule, ɵɵdefineInjector, ɵɵsetNgModuleScope, NgModule } from '@angular/core';
import { Subject, share, Observable } from 'rxjs';
import { EventManager } from '@angular/platform-browser';
import { DOCUMENT, NgClass, NgForOf, CommonModule } from '@angular/common';

const HOTKEY_OPTIONS = new InjectionToken('HotkeyOptions');

const _defaultOptions = {
    disableCheatSheet: false,
    cheatSheetTitle: 'Keyboard Shortcuts:',
    cheatSheetHotkey: 'shift.?',
    cheatSheetHotkeyDescription: 'Show / hide this help menu',
    cheatSheetCloseEsc: false,
    cheatSheetCloseEscDescription: 'Hide this help menu'
};
class NgxHotkeysService {
    constructor(eventManager, config, doc) {
        this.eventManager = eventManager;
        this.config = config;
        this.registeredHotkeys = new Set();
        this.pausedHotkeys = new Set();
        this._cheatSheetToggled = new Subject();
        this.bindingSubscriptions = new Map();
        this.document = doc;
        this.defaults = { element: this.document };
        this.configuration = Object.assign(Object.assign({}, _defaultOptions), this.config);
        if (!this.configuration.disableCheatSheet) {
            this.register({
                combo: this.configuration.cheatSheetHotkey,
                handler: () => {
                    this._cheatSheetToggled.next(true);
                },
                description: this.configuration.cheatSheetHotkeyDescription
            });
        }
        if (this.configuration.cheatSheetCloseEsc) {
            this.register({
                combo: 'esc',
                handler: () => {
                    this._cheatSheetToggled.next(false);
                },
                description: this.configuration.cheatSheetCloseEscDescription
            });
        }
    }
    /**
     * Returns the registered hotkeys as array.
     * @returns all registered hotkeys
     */
    get hotkeys() {
        return Array.from(this.registeredHotkeys);
    }
    /**
     * Returns an Observable stream indicating the cheatsheets visibility was toggled.
     * @returns stream indicating the cheatsheets visibility was toggled
     */
    get cheatSheetToggled() {
        return this._cheatSheetToggled.asObservable().pipe(share());
    }
    /**
     * Returns the options used to configure this service.
     */
    get options() {
        return this.configuration;
    }
    /**
     * Registers a new hotkey/new hotkeys with it's/their handler(s).
     * @param hotkey hotkeys to listen for
     * @param unpausing flag indicating if the hotkeys should be unpaused
     */
    register(hotkey, unpausing = false) {
        let hotkeys = [].concat(hotkey);
        if (unpausing) {
            hotkeys = Array.from(this.pausedHotkeys);
        }
        hotkeys.forEach(h => {
            if (unpausing) {
                this.pausedHotkeys.delete(h);
            }
            this.registeredHotkeys.add(h);
            this.bindingSubscriptions.set(h.combo, this.bindToEventManager(h)
                .subscribe(() => {
                h.handler();
            }));
        });
    }
    /**
     * Removes a/the registered hotkey(s).
     * @param hotkey hotkey filter
     * @param pausing flag indicating if the hotkeys should be paused
     */
    unregister(hotkey, pausing = false) {
        const hotkeys = [].concat(hotkey);
        hotkeys.forEach(h => {
            this.registeredHotkeys.delete(h);
            if (pausing) {
                this.pausedHotkeys.add(h);
            }
            this.unbindFromEventManager(h);
        });
    }
    /**
     * Returns all hotkeys matching the passed combo(s).
     * @param combo combo to match against
     * @returns all matched hotkeys
     */
    get(combo) {
        return this.hotkeys.filter(h => h.combo === combo);
    }
    /**
     * Stops listening for the specified hotkeys.
     * @param hotkey hotkey filter
     */
    pause(hotkey) {
        if (!hotkey) {
            return this.pause(this.hotkeys);
        }
        const hotkeys = [].concat(hotkey);
        this.unregister(hotkeys, true);
    }
    /**
     * Resumes listening for the specified hotkeys.
     * @param hotkey hotkey filter
     */
    unpause(hotkey) {
        if (!hotkey) {
            return this.unpause(this.hotkeys);
        }
        const hotkeys = [].concat(hotkey);
        this.register(hotkeys, true);
    }
    /**
     * Resets all hotkeys.
     */
    reset() {
        this.registeredHotkeys.forEach((h) => this.unbindFromEventManager(h));
        this.registeredHotkeys.clear();
        this.pausedHotkeys.clear();
    }
    ngOnDestroy() {
        this.reset();
    }
    bindToEventManager(hotkey) {
        const merged = Object.assign(Object.assign({}, this.defaults), hotkey);
        const event = `keydown.${merged.combo.replace('+', '.')}`;
        return new Observable(observer => {
            const handler = (e) => {
                e.preventDefault();
                observer.next(e);
            };
            const dispose = this.eventManager.addEventListener(merged.element, event, handler);
            return () => {
                dispose();
            };
        });
    }
    unbindFromEventManager(hotkey) {
        const merged = Object.assign(Object.assign({}, this.defaults), hotkey);
        const keySub = this.bindingSubscriptions.get(merged.combo);
        if (keySub && !keySub.closed) {
            keySub.unsubscribe();
            this.bindingSubscriptions.delete(merged.combo);
        }
    }
}
NgxHotkeysService.ɵfac = function NgxHotkeysService_Factory(t) { return new (t || NgxHotkeysService)(ɵɵinject(EventManager), ɵɵinject(HOTKEY_OPTIONS), ɵɵinject(DOCUMENT)); };
NgxHotkeysService.ɵprov = ɵɵdefineInjectable({ token: NgxHotkeysService, factory: NgxHotkeysService.ɵfac, providedIn: 'root' });
/*@__PURE__*/ (function () { ɵsetClassMetadata(NgxHotkeysService, [{
        type: Injectable,
        args: [{
                providedIn: 'root'
            }]
    }], function () { return [{ type: EventManager }, { type: undefined, decorators: [{
                type: Inject,
                args: [HOTKEY_OPTIONS]
            }] }, { type: undefined, decorators: [{
                type: Inject,
                args: [DOCUMENT]
            }] }]; }, null); })();

function NgxCheatsheetComponent_tr_6_Template(rf, ctx) { if (rf & 1) {
    ɵɵelementStart(0, "tr");
    ɵɵelementStart(1, "td", 5);
    ɵɵelementStart(2, "span", 6);
    ɵɵtext(3);
    ɵɵelementEnd();
    ɵɵelementEnd();
    ɵɵelementStart(4, "td", 7);
    ɵɵtext(5);
    ɵɵelementEnd();
    ɵɵelementEnd();
} if (rf & 2) {
    const hotkey_r1 = ctx.$implicit;
    const ctx_r0 = ɵɵnextContext();
    ɵɵadvance(3);
    ɵɵtextInterpolate(ctx_r0.formatHotkey(hotkey_r1));
    ɵɵadvance(2);
    ɵɵtextInterpolate(hotkey_r1.description);
} }
const _c0 = function (a0) { return { "in": a0 }; };
class NgxCheatsheetComponent {
    constructor(hotkeysService) {
        this.hotkeysService = hotkeysService;
        this.helpVisible = false;
        this._map = {
            meta: '\u2318',
            shift: '\u21E7',
            left: '\u2190',
            right: '\u2192',
            up: '\u2191',
            down: '\u2193',
            'return': '\u23CE',
            backspace: '\u232B' // ⌫
        };
        this.title = this.hotkeysService.options.cheatSheetTitle || this.title;
    }
    ngOnInit() {
        this._subscription = this.hotkeysService.cheatSheetToggled
            .subscribe((isOpen) => {
            if (isOpen !== false) {
                this.hotkeys = this.hotkeysService.hotkeys
                    .filter(hotkey => hotkey.description);
            }
            if (isOpen === false) {
                this.helpVisible = false;
            }
            else {
                this.toggleCheatSheet();
            }
        });
    }
    formatHotkey(hotkey) {
        const combo = Array.isArray(hotkey.combo) ? hotkey.combo[0] : hotkey.combo;
        const sequence = combo.split(/[\s]/);
        for (let i = 0; i < sequence.length; i++) {
            sequence[i] = this.symbolize(sequence[i]);
        }
        return sequence;
    }
    symbolize(combo) {
        const comboSplit = combo.split('.');
        for (let i = 0; i < comboSplit.length; i++) {
            // try to resolve command / ctrl based on OS:
            if (comboSplit[i] === 'mod') {
                if (window.navigator && window.navigator.platform.indexOf('Mac') >= 0) {
                    comboSplit[i] = 'meta';
                }
                else {
                    comboSplit[i] = 'ctrl';
                }
            }
            comboSplit[i] = this._map[comboSplit[i]] || comboSplit[i];
        }
        return comboSplit.join(' + ');
    }
    ngOnDestroy() {
        if (this._subscription && !this._subscription.closed) {
            this._subscription.unsubscribe();
        }
    }
    toggleCheatSheet() {
        this.helpVisible = !this.helpVisible;
    }
}
NgxCheatsheetComponent.ɵfac = function NgxCheatsheetComponent_Factory(t) { return new (t || NgxCheatsheetComponent)(ɵɵdirectiveInject(NgxHotkeysService)); };
NgxCheatsheetComponent.ɵcmp = ɵɵdefineComponent({ type: NgxCheatsheetComponent, selectors: [["ngx-cheatsheet"]], inputs: { title: "title" }, decls: 9, vars: 5, consts: [[1, "ngx-cheatsheet", "fade", 2, "display", "none", 3, "ngClass"], [1, "ngx-cheatsheet__hotkeys"], [1, "ngx-cheatsheet__hotkeys__title"], [4, "ngFor", "ngForOf"], [1, "ngx-cheatsheet__hotkeys__close", 3, "click"], [1, "ngx-cheatsheet__hotkeys__keys"], [1, "ngx-cheatsheet__hotkeys__keys__key"], [1, "ngx-cheatsheet__hotkeys__keys__text"]], template: function NgxCheatsheetComponent_Template(rf, ctx) { if (rf & 1) {
        ɵɵelementStart(0, "div", 0);
        ɵɵelementStart(1, "div", 1);
        ɵɵelementStart(2, "h4", 2);
        ɵɵtext(3);
        ɵɵelementEnd();
        ɵɵelementStart(4, "table");
        ɵɵelementStart(5, "tbody");
        ɵɵtemplate(6, NgxCheatsheetComponent_tr_6_Template, 6, 2, "tr", 3);
        ɵɵelementEnd();
        ɵɵelementEnd();
        ɵɵelementStart(7, "div", 4);
        ɵɵlistener("click", function NgxCheatsheetComponent_Template_div_click_7_listener() { return ctx.toggleCheatSheet(); });
        ɵɵtext(8, "\u00D7");
        ɵɵelementEnd();
        ɵɵelementEnd();
        ɵɵelementEnd();
    } if (rf & 2) {
        ɵɵproperty("ngClass", ɵɵpureFunction1(3, _c0, ctx.helpVisible));
        ɵɵadvance(3);
        ɵɵtextInterpolate(ctx.title);
        ɵɵadvance(3);
        ɵɵproperty("ngForOf", ctx.hotkeys);
    } }, directives: [NgClass, NgForOf], styles: [".ngx-cheatsheet[_ngcontent-%COMP%]{display:table!important;position:fixed;width:100%;height:100%;top:0;left:0;color:#333;font-size:1em;background-color:rgba(255,255,255,.9)}.ngx-cheatsheet.fade[_ngcontent-%COMP%]{z-index:-1024;visibility:hidden;opacity:0;transition:opacity .15s linear}.ngx-cheatsheet.fade.in[_ngcontent-%COMP%]{z-index:10002;visibility:visible;opacity:1}.ngx-cheatsheet__hotkeys[_ngcontent-%COMP%]{width:100%;height:100%;display:table-cell;vertical-align:middle}.ngx-cheatsheet__hotkeys__title[_ngcontent-%COMP%]{font-weight:700;text-align:center;font-size:1.2em}.ngx-cheatsheet__hotkeys[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]{margin:auto;color:#333}.ngx-cheatsheet__hotkeys__keys[_ngcontent-%COMP%]{padding:5px;text-align:right}.ngx-cheatsheet__hotkeys__keys__key[_ngcontent-%COMP%]{display:inline-block;color:#fff;background-color:#333;border:1px solid #333;border-radius:5px;text-align:center;margin-right:5px;box-shadow:inset 0 1px 0 #666,0 1px 0 #bbb;padding:5px 9px;font-size:1em}.ngx-cheatsheet__hotkeys__keys__text[_ngcontent-%COMP%]{padding-left:10px;font-size:1em}.ngx-cheatsheet__hotkeys__close[_ngcontent-%COMP%]{position:fixed;top:20px;right:20px;font-size:2em;font-weight:700;padding:5px 10px;border:1px solid #ddd;border-radius:5px;min-height:45px;min-width:45px;text-align:center;cursor:pointer}@media all and (max-width:500px){.ngx-cheatsheet__hotkeys[_ngcontent-%COMP%]{font-size:.8em}}@media all and (min-width:750px){.ngx-cheatsheet__hotkeys[_ngcontent-%COMP%]{font-size:1.2em}}"] });
/*@__PURE__*/ (function () { ɵsetClassMetadata(NgxCheatsheetComponent, [{
        type: Component,
        args: [{
                selector: 'ngx-cheatsheet',
                templateUrl: './ngx-cheatsheet.component.html',
                styleUrls: ['./ngx-cheatsheet.component.scss']
            }]
    }], function () { return [{ type: NgxHotkeysService }]; }, { title: [{
            type: Input
        }] }); })();

class NgxHotkeysModule {
    static forRoot(options) {
        return {
            ngModule: NgxHotkeysModule,
            providers: [
                NgxHotkeysService,
                {
                    provide: HOTKEY_OPTIONS,
                    useValue: options
                }
            ]
        };
    }
}
NgxHotkeysModule.ɵmod = ɵɵdefineNgModule({ type: NgxHotkeysModule });
NgxHotkeysModule.ɵinj = ɵɵdefineInjector({ factory: function NgxHotkeysModule_Factory(t) { return new (t || NgxHotkeysModule)(); }, imports: [[
            CommonModule
        ]] });
(function () { (typeof ngJitMode === "undefined" || ngJitMode) && ɵɵsetNgModuleScope(NgxHotkeysModule, { declarations: [NgxCheatsheetComponent], imports: [CommonModule], exports: [NgxCheatsheetComponent] }); })();
/*@__PURE__*/ (function () { ɵsetClassMetadata(NgxHotkeysModule, [{
        type: NgModule,
        args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    NgxCheatsheetComponent
                ],
                exports: [
                    NgxCheatsheetComponent
                ]
            }]
    }], null, null); })();

/**
 * Generated bundle index. Do not edit.
 */

export { NgxCheatsheetComponent, NgxHotkeysModule, NgxHotkeysService };
//# sourceMappingURL=ngx-hotkeys-vnc.js.map
