import { OnDestroy } from '@angular/core';
import { Observable } from 'rxjs';
import { Hotkey, HotkeyOptions } from '../interfaces';
import { EventManager } from '@angular/platform-browser';
export declare class NgxHotkeysService implements OnDestroy {
    private eventManager;
    private config?;
    private document?;
    private configuration;
    private registeredHotkeys;
    private pausedHotkeys;
    private _cheatSheetToggled;
    private bindingSubscriptions;
    private defaults;
    constructor(eventManager: EventManager, config?: HotkeyOptions, doc?: any);
    /**
     * Returns the registered hotkeys as array.
     * @returns all registered hotkeys
     */
    get hotkeys(): Hotkey[];
    /**
     * Returns an Observable stream indicating the cheatsheets visibility was toggled.
     * @returns stream indicating the cheatsheets visibility was toggled
     */
    get cheatSheetToggled(): Observable<boolean>;
    /**
     * Returns the options used to configure this service.
     */
    get options(): HotkeyOptions;
    /**
     * Registers a new hotkey/new hotkeys with it's/their handler(s).
     * @param hotkey hotkeys to listen for
     * @param unpausing flag indicating if the hotkeys should be unpaused
     */
    register(hotkey: Hotkey | Hotkey[], unpausing?: boolean): void;
    /**
     * Removes a/the registered hotkey(s).
     * @param hotkey hotkey filter
     * @param pausing flag indicating if the hotkeys should be paused
     */
    unregister(hotkey: Hotkey | Hotkey[], pausing?: boolean): void;
    /**
     * Returns all hotkeys matching the passed combo(s).
     * @param combo combo to match against
     * @returns all matched hotkeys
     */
    get(combo?: string): Hotkey[];
    /**
     * Stops listening for the specified hotkeys.
     * @param hotkey hotkey filter
     */
    pause(hotkey?: Hotkey | Hotkey[]): void;
    /**
     * Resumes listening for the specified hotkeys.
     * @param hotkey hotkey filter
     */
    unpause(hotkey?: Hotkey | Hotkey[]): void;
    /**
     * Resets all hotkeys.
     */
    reset(): void;
    ngOnDestroy(): void;
    private bindToEventManager;
    private unbindFromEventManager;
}
